#jpo-welcome-panel {
  background-repeat: no-repeat;
  background-position: center bottom;
  -webkit-background-size: 520px auto;
  position: fixed;
  top: 0;
  left: 160px;
  width: 100%;
  height: 100%;
  overflow: auto;
  box-sizing: border-box;
  background-color: #fff;
  padding: 100px;
  z-index: 999;
  margin-bottom: 20px; }
  #jpo-welcome-panel *, #jpo-welcome-panel *:before, #jpo-welcome-panel *:after {
    box-sizing: border-box; }
  @media (max-width: 960px) {
    #jpo-welcome-panel {
      left: 36px; } }
  @media (max-width: 782px) {
    #jpo-welcome-panel {
      left: 0px; } }
  #jpo-welcome-panel .submit {
    border-radius: 6px; }
  #jpo-welcome-panel #dopsid0 .submit {
    padding: 10px !important;
    margin-top: 25px !important;
    margin-bottom: 25px !important; }

.welcome-panel {
  overflow: visible !important; }

.getting-started {
  padding-bottom: 10px; }
  .getting-started h4 {
    margin: 0;
    font-size: 20px; }

#welcome__site-title input {
  width: 100%; }

.welcome__content_box {
  background: white;
  margin: 10px;
  border: 1px solid #eee; }
  .welcome__content_box h3 {
    padding: 5px 5px 10px 10px;
    border-bottom: 1px solid #eee; }

.welcome__section h3 em {
  font-weight: bold; }

.welcome__section h4 {
  border-bottom: 1px #ddd solid; }

#welcome__stats a.skip, #welcome__traffic a.skip {
  margin-left: 0; }

.welcome__helper {
  padding-top: 15px;
  border-top: 1px solid #ddd; }

#welcome__design .theme-browser {
  margin-top: 10px;
  padding-left: 10px;
  min-width: 180px; }
  #welcome__design .theme-browser .theme.active {
    box-shadow: 0px 0px 8px 0px #3592b6;
    border: 1px solid #3592b6; }
  #welcome__design .theme-browser .theme-name {
    padding-right: 0px; }
  @media (min-width: 782px) {
    #welcome__design .theme-browser .theme {
      width: 30%; } }
  @media (max-width: 600px) {
    #welcome__design .theme-browser .theme {
      display: block;
      float: none; } }
  #welcome__design .theme-browser * {
    box-sizing: content-box; }
  #welcome__design .theme-browser h3 {
    line-height: 1.4em;
    text-align: left !important; }

#welcome__design .wp-pointer-content h3:before {
  content: '\f100'; }

/*# sourceMappingURL=welcome-panel.css.map */
